% Copyright 2018. All Rights Reserved
% Code by Shuang Li
% For Paper, "Atomic Norm Minimization for Modal Analysis from Random and Compressed Samples"
% IEEE transactions on signal processing, 2018
% by Shuang Li, Dehui Yang, Gongguo Tang, and Michael B. Wakin
% This code is used to reproduce Figure 7.

%% generate data
clc; clear; close all;

KK = 6;
m = 1:KK;
m = m';
M = diag(m);     % mass matrix
k1 = 500; k2 = 300; k3 = 100; k4 = 50; k5 = 400; k6 = 200; k7 = 500;
K = [k1+k2 -k2 0 0 0 0; -k2 k2+k3 -k3 0 0 0; 0 -k3 k3+k4 -k4 0 0; 0 0 -k4 k4+k5 -k5 0;...
    0 0 0 -k5 k5+k6 -k6; 0 0 0 0 -k6 k6+k7];   % stiffness matrix
% Proportional damping
alpha = 0.01;
beta = 0.006;

[mshape, S] = eig(K,M);
mshape = mshape*diag(1./norms(mshape));
Ktilde = mshape'*K*mshape;
Mtilde = mshape'*M*mshape;
Ctilde = alpha*Mtilde+beta*Ktilde;

xi = (diag(Ctilde))./(2*sqrt(diag(Mtilde).*diag(Ktilde)));  % damping ratios
Freq = sqrt(diag(Ktilde)./diag(Mtilde));   % natural frequencies
freq = sort(Freq/2/pi);                    % Hz
freq_damp = freq.*sqrt(ones(KK,1)-xi.^2);  % damped natural frequencies
A = [1 0.85 0.7 0.5 0.25 0.7];             % amplitudes
Nsensor=size(mshape,1);                    % number of sensors
Nshape=size(mshape,2);                     % number of active modes
Nsample=300;                               % number of uniform samples
Ts=0.5/max(freq);                          % sampling intervals
Time=0:Ts:Ts*(Nsample-1);
QQ = exp(-2*pi*(xi.*freq)*Time).*cos(2*pi*freq_damp*Time);      % real

Datac=mshape*diag(A)*QQ;   
Datac= transpose(Datac);  % real data
Data = hilbert(Datac);    % obtain analytic samples with Hilbert transform

%% Show the real data we have
figure
lw = 3;
fs=24;     % FontSize
subplot(6,1,1)
plot(Datac(:,1),'b-','Linewidth',lw)
ylabel('Node 1','FontSize',fs)
set(gca,'FontSize',fs);

subplot(6,1,2)
plot(Datac(:,2),'b-','Linewidth',lw)
ylabel('Node 2','FontSize',fs)
set(gca,'FontSize',fs);

subplot(6,1,3)
plot(Datac(:,3),'b-','Linewidth',lw)
ylabel('Node 3','FontSize',fs)
set(gca,'FontSize',fs);

subplot(6,1,4)
plot(Datac(:,4),'b-','Linewidth',lw)
ylabel('Node 4','FontSize',fs)
set(gca,'FontSize',fs);

subplot(6,1,5)
plot(Datac(:,5),'b-','Linewidth',lw)
ylabel('Node 5','FontSize',fs)
set(gca,'FontSize',fs);

subplot(6,1,6)
plot(Datac(:,6),'b-','Linewidth',lw)
ylabel('Node 6','FontSize',fs)
xlabel('Time','FontSize',fs)
set(gca,'FontSize',fs);


ss1 = 1; ss2 = 100; Nsample = ss2-ss1+1;
Data = Data(ss1:ss2,:);

%% Atomic Norm Minimization

cvx_begin quiet
variable u0;
variable u(Nsample-1) complex;
variable Tu(Nsample,Nsample) hermitian;
variable X(Nsample,Nsensor) complex;
variable W(Nsensor,Nsensor) Hermitian;
dual variable Y0
minimize(0.5/Nsample*trace(Tu)+0.5*trace(W))
subject to
Tu == toeplitz([u0;conj(u)],[u0;conj(u)]');
Y0:[Tu   X; ...
    X'   W] == semidefinite(Nsample+Nsensor,Nsample+Nsensor);
X==Data;
cvx_end

Y=-2*Y0(1:Nsample,Nsample+1:Nsample+Nsensor);   % dual solution
f=0:0.0001:5;
Time=0:Ts:Ts*(Nsample-1);
a=exp(1j*2*pi*Time'*f);
Q=Y'*a;      % dual polynomial
if Nsensor > 1
    NormQ=norms(Q);
else
    NormQ=abs(Q);
end

[pks,locs] = findpeaks(NormQ,'minpeakdistance',1,'MinPeakHeight',0.99);
Freq_est=f(locs);      % estimated frequencies

Af=exp(1j*2*pi*Time'*Freq_est);
Phi=Af\X;
Phi=transpose(Phi);    % estimated mode shapes
A_est=norms(Phi);      % estimated amplitudes

if length(A_est) > KK
    [A_est_sort, idx] = sort(A_est,'descend');
    Freq_est(idx(KK+1:end)) = [];    % estimated frequencies
end

Af=exp(1j*2*pi*Time'*Freq_est);
Phi=Af\X;
Phi=transpose(Phi);                 
A_est=norms(Phi);                   % estimated amplitudes

MS_estimate = Phi*diag(1./A_est);   % estimated mode shapes
MAC=diag(abs(mshape'*MS_estimate));  % compute MAC


%% SVD
[u,s,~] = svd(transpose(Data));  % !!!!!!
MShape_SVD=u;
MAC_SVD=abs(mshape'*MShape_SVD);
idx1=find(MAC_SVD(1,:)==max(MAC_SVD(1,:)));
idx2=find(MAC_SVD(2,:)==max(MAC_SVD(2,:)));
idx3=find(MAC_SVD(3,:)==max(MAC_SVD(3,:)));
idx4=find(MAC_SVD(4,:)==max(MAC_SVD(4,:)));
idx5=find(MAC_SVD(5,:)==max(MAC_SVD(5,:)));
idx6=find(MAC_SVD(6,:)==max(MAC_SVD(6,:)));
MShape_SVD=MShape_SVD(:,[idx1 idx2 idx3 idx4 idx5 idx6]);      % recovered modal shape
MAC_SVD=diag(abs(mshape'*MShape_SVD));

%% Show results
figure
lw=3;
fs=24;
subplot(3,2,1)
plot(real(mshape(:,1)),'r-','Linewidth',lw);hold on;
plot(real(MS_estimate(:,1)),'b--','Linewidth',lw);hold on;
plot(real(MShape_SVD(:,1)),'g-.','Linewidth',lw);hold on;
ylabel('Mode Shape 1','FontSize',fs)
set(gca,'FontSize',fs);
axis tight;

subplot(3,2,2)
plot(real(mshape(:,2)),'r-','Linewidth',lw);hold on;
plot(real(MS_estimate(:,2)),'b--','Linewidth',lw);hold on;
plot(real(MShape_SVD(:,2)),'g-.','Linewidth',lw);hold on;
ylabel('Mode Shape 2','FontSize',fs)
set(gca,'FontSize',fs);
axis tight;

subplot(3,2,3)
plot(real(mshape(:,3)),'r-','Linewidth',lw);hold on;
plot(real(MS_estimate(:,3)),'b--','Linewidth',lw);hold on;
plot(real(-MShape_SVD(:,3)),'g-.','Linewidth',lw);hold on;
ylabel('Mode Shape 3','FontSize',fs)
set(gca,'FontSize',fs);
axis tight;


subplot(3,2,4)
plot(real(mshape(:,4)),'r-','Linewidth',lw);hold on;
plot(real(MS_estimate(:,4)),'b--','Linewidth',lw);hold on;
plot(real(MShape_SVD(:,4)),'g-.','Linewidth',lw);hold on;
ylabel('Mode Shape 4','FontSize',fs)
set(gca,'FontSize',fs);
axis tight;

subplot(3,2,5)
plot(real(mshape(:,5)),'r-','Linewidth',lw);hold on;
plot(real(MS_estimate(:,5)),'b--','Linewidth',lw);hold on;
plot(real(MShape_SVD(:,5)),'g-.','Linewidth',lw);hold on;
xlabel('Node','FontSize',fs)
ylabel('Mode Shape 5','FontSize',fs)
set(gca,'FontSize',fs);
axis tight;

subplot(3,2,6)
plot(real(mshape(:,6)),'r-','Linewidth',lw);hold on;
plot(real(MS_estimate(:,6)),'b--','Linewidth',lw);hold on;
plot(real(MShape_SVD(:,6)),'g-.','Linewidth',lw);hold on;
xlabel('Node','FontSize',fs)
ylabel('Mode Shape 6','FontSize',fs)
legend('True', 'ANM', 'SVD')
set(gca,'FontSize',fs);
axis tight;


